@@ -1,6 +1,6 @@
 /* engine-gpgsm.c - GpgSM engine.
    Copyright (C) 2000 Werner Koch (dd9jn)
-   Copyright (C) 2001, 2002, 2003 g10 Code GmbH
+   Copyright (C) 2001, 2002, 2003, 2004 g10 Code GmbH
  
    This file is part of GPGME.
 
@@ -40,6 +40,7 @@
 
 #include "assuan.h"
 #include "status-table.h"
+#include "debug.h"
 
 #include "engine-backend.h"
 
@@ -152,6 +153,12 @@ close_notify_handler (int fd, void *opaque)
 static gpgme_error_t
 map_assuan_error (AssuanError err)
 {
+  if (!err)
+    return 0;
+
+  if (err == -1)
+    return gpg_error (GPG_ERR_INV_ENGINE);
+
   /* New code will use gpg_error_t values.  */
   if (gpg_err_source (err))
     return (gpgme_error_t) err;
@@ -269,13 +276,13 @@ map_assuan_error (AssuanError err)
 }
 
 
-static void
-gpgsm_release (void *engine)
+static gpgme_error_t
+gpgsm_cancel (void *engine)
 {
   engine_gpgsm_t gpgsm = engine;
 
   if (!gpgsm)
-    return;
+    return gpg_error (GPG_ERR_INV_VALUE);
 
   if (gpgsm->status_cb.fd != -1)
     _gpgme_io_close (gpgsm->status_cb.fd);
@@ -286,7 +293,25 @@ gpgsm_release (void *engine)
   if (gpgsm->message_cb.fd != -1)
     _gpgme_io_close (gpgsm->message_cb.fd);
 
-  assuan_disconnect (gpgsm->assuan_ctx);
+  if (gpgsm->assuan_ctx)
+    {
+      assuan_disconnect (gpgsm->assuan_ctx);
+      gpgsm->assuan_ctx = NULL;
+    }
+
+  return 0;
+}
+
+
+static void
+gpgsm_release (void *engine)
+{
+  engine_gpgsm_t gpgsm = engine;
+
+  if (!gpgsm)
+    return;
+
+  gpgsm_cancel (engine);
 
   free (gpgsm->colon.attic.line);
   free (gpgsm);
@@ -294,7 +319,7 @@ gpgsm_release (void *engine)
 
 
 static gpgme_error_t
-gpgsm_new (void **engine)
+gpgsm_new (void **engine, const char *lc_ctype, const char *lc_messages)
 {
   gpgme_error_t err = 0;
   engine_gpgsm_t gpgsm;
@@ -302,10 +327,8 @@ gpgsm_new (void **engine)
   int fds[2];
   int child_fds[4];
   char *dft_display = NULL;
-  char *dft_ttyname = NULL;
+  char dft_ttyname[64];
   char *dft_ttytype = NULL;
-  char *old_lc = NULL;
-  char *dft_lc = NULL;
   char *optstr;
   int fdlist[5];
   int nfds;
@@ -378,6 +401,7 @@ gpgsm_new (void **engine)
 
   err = assuan_pipe_connect (&gpgsm->assuan_ctx,
 			     _gpgme_get_gpgsm_path (), argv, child_fds);
+  /* FIXME: Check error.  */
 
   /* We need to know the fd used by assuan for reads.  We do this by
      using the assumption that the first returned fd from
@@ -403,14 +427,19 @@ gpgsm_new (void **engine)
   gpgsm->status_cb.dir = 1;
   gpgsm->status_cb.data = gpgsm;
 
-  dft_display = getenv ("DISPLAY");
+  err = _gpgme_getenv ("DISPLAY", &dft_display);
+  if (err)
+    goto leave;
   if (dft_display)
     {
       if (asprintf (&optstr, "OPTION display=%s", dft_display) < 0)
         {
+	  free (dft_display);
 	  err = gpg_error_from_errno (errno);
 	  goto leave;
 	}
+      free (dft_display);
+
       err = assuan_transact (gpgsm->assuan_ctx, optstr, NULL, NULL, NULL,
 			     NULL, NULL, NULL);
       free (optstr);
@@ -420,27 +449,17 @@ gpgsm_new (void **engine)
 	  goto leave;
 	}
     }
-  dft_ttyname = ttyname (1);
-  if (dft_ttyname)
+
+  if (isatty (1))
     {
-      if (asprintf (&optstr, "OPTION ttyname=%s", dft_ttyname) < 0)
-        {
+      if (ttyname_r (1, dft_ttyname, sizeof (dft_ttyname)))
+	{
 	  err = gpg_error_from_errno (errno);
 	  goto leave;
 	}
-      err = assuan_transact (gpgsm->assuan_ctx, optstr, NULL, NULL, NULL,
-			     NULL, NULL, NULL);
-      free (optstr);
-      if (err)
+      else
 	{
-	  err = map_assuan_error (err);
-	  goto leave;
-	}
-
-      dft_ttytype = getenv ("TERM");
-      if (dft_ttytype)
-	{
-	  if (asprintf (&optstr, "OPTION ttytype=%s", dft_ttytype) < 0)
+	  if (asprintf (&optstr, "OPTION ttyname=%s", dft_ttyname) < 0)
 	    {
 	      err = gpg_error_from_errno (errno);
 	      goto leave;
@@ -453,82 +472,73 @@ gpgsm_new (void **engine)
 	      err = map_assuan_error (err);
 	      goto leave;
 	    }
-	}
 
-      old_lc = setlocale (LC_CTYPE, NULL);
-      if (old_lc)
-        {
-          old_lc = strdup (old_lc);
-          if (!old_lc)
-            {
-              err = gpg_error_from_errno (errno);
-              goto leave;
-            }
-        }
-      dft_lc = setlocale (LC_CTYPE, "");
-      if (dft_lc)
-	{
-	  if (asprintf (&optstr, "OPTION lc-ctype=%s", dft_lc) < 0)
-	    err = gpg_error_from_errno (errno);
-	  else
+	  err = _gpgme_getenv ("TERM", &dft_ttytype);
+	  if (err)
+	    goto leave;
+	  if (dft_ttytype)
 	    {
-	      err = assuan_transact (gpgsm->assuan_ctx, optstr, NULL, NULL,
-                                     NULL, NULL, NULL, NULL);
-	      free (optstr);
-	      if (err)
-		err = map_assuan_error (err);
-	    }
-	}
-      if (old_lc)
-        {
-          setlocale (LC_CTYPE, old_lc);
-          free (old_lc);
-        }
-      if (err)
-	goto leave;
+	      if (asprintf (&optstr, "OPTION ttytype=%s", dft_ttytype) < 0)
+		{
+		  free (dft_ttytype);
+		  err = gpg_error_from_errno (errno);
+		  goto leave;
+		}
+	      free (dft_ttytype);
 
-      old_lc = setlocale (LC_MESSAGES, NULL);
-      if (old_lc)
-        {
-          old_lc = strdup (old_lc);
-          if (!old_lc)
-            {
-              err = gpg_error_from_errno (errno);
-              goto leave;
-            }
-        }
-      dft_lc = setlocale (LC_MESSAGES, "");
-      if (dft_lc)
-	{
-	  if (asprintf (&optstr, "OPTION lc-messages=%s", dft_lc) < 0)
-	    err = gpg_error_from_errno (errno);
-	  else
-	    {
 	      err = assuan_transact (gpgsm->assuan_ctx, optstr, NULL, NULL,
 				     NULL, NULL, NULL, NULL);
 	      free (optstr);
 	      if (err)
-		err = map_assuan_error (err);
+		{
+		  err = map_assuan_error (err);
+		  goto leave;
+		}
 	    }
 	}
-      if (old_lc)
-        {
-          setlocale (LC_MESSAGES, old_lc);
-          free (old_lc);
-        }
-      if (err)
-	goto leave;
     }
 
-  if (!err &&
-      (_gpgme_io_set_close_notify (gpgsm->status_cb.fd,
-				   close_notify_handler, gpgsm)
-       || _gpgme_io_set_close_notify (gpgsm->input_cb.fd,
-				   close_notify_handler, gpgsm)
-       || _gpgme_io_set_close_notify (gpgsm->output_cb.fd,
+  if (lc_ctype)
+    {
+      if (asprintf (&optstr, "OPTION lc-ctype=%s", lc_ctype) < 0)
+	err = gpg_error_from_errno (errno);
+      else
+	{
+	  err = assuan_transact (gpgsm->assuan_ctx, optstr, NULL, NULL,
+				 NULL, NULL, NULL, NULL);
+	  free (optstr);
+	  if (err)
+	    err = map_assuan_error (err);
+	}
+    }
+  if (err)
+    goto leave;
+  
+  if (lc_messages)
+    {
+      if (asprintf (&optstr, "OPTION lc-messages=%s", lc_messages) < 0)
+	err = gpg_error_from_errno (errno);
+      else
+	{
+	  err = assuan_transact (gpgsm->assuan_ctx, optstr, NULL, NULL,
+				 NULL, NULL, NULL, NULL);
+	  free (optstr);
+	  if (err)
+	    err = map_assuan_error (err);
+	}
+    }
+  if (err)
+    goto leave;
+
+  if (!err
+      && (_gpgme_io_set_close_notify (gpgsm->status_cb.fd,
 				      close_notify_handler, gpgsm)
-       || _gpgme_io_set_close_notify (gpgsm->message_cb.fd,
-				      close_notify_handler, gpgsm)))
+	  || _gpgme_io_set_close_notify (gpgsm->input_cb.fd,
+					 close_notify_handler, gpgsm)
+	  || _gpgme_io_set_close_notify (gpgsm->output_cb.fd,
+					 close_notify_handler, gpgsm)
+	  || _gpgme_io_set_close_notify (gpgsm->message_cb.fd,
+					 close_notify_handler, gpgsm)))
     {
       err = gpg_error (GPG_ERR_GENERAL);
       goto leave;
@@ -601,7 +611,7 @@ gpgsm_assuan_simple_command (ASSUAN_CONTEXT ctx, char *cmd,
 	  r = parse_status (line + 2);
 
 	  if (r >= 0 && status_fnc)
-	    status_fnc (status_fnc_value, r, rest);
+	    err = status_fnc (status_fnc_value, r, rest);
 	  else
 	    err = gpg_error (GPG_ERR_GENERAL);
 	}
@@ -685,8 +695,10 @@ status_handler (void *opaque, int fd)
       if (assuan_err)
 	{
 	  /* Try our best to terminate the connection friendly.  */
-	  assuan_write_line (gpgsm->assuan_ctx, "BYE");
+	  /*	  assuan_write_line (gpgsm->assuan_ctx, "BYE"); */
 	  err = map_assuan_error (assuan_err);
+          DEBUG3 ("fd %d: error from assuan (%d) getting status line : %s \n",
+                  fd, assuan_err, gpg_strerror (err));
 	}
       else if (linelen >= 3
 	       && line[0] == 'E' && line[1] == 'R' && line[2] == 'R'
@@ -696,6 +708,8 @@ status_handler (void *opaque, int fd)
 	    err = map_assuan_error (atoi (&line[4]));
 	  else
 	    err = gpg_error (GPG_ERR_GENERAL);
+          DEBUG2 ("fd %d: ERR line - mapped to: %s\n",
+                  fd, err? gpg_strerror (err):"ok");
 	}
       else if (linelen >= 2
 	       && line[0] == 'O' && line[1] == 'K'
@@ -716,6 +730,8 @@ status_handler (void *opaque, int fd)
               err = gpgsm->colon.fnc (gpgsm->colon.fnc_value, NULL);
             }
 	  _gpgme_io_close (gpgsm->status_cb.fd);
+          DEBUG2 ("fd %d: OK line - final status: %s\n",
+                  fd, err? gpg_strerror (err):"ok");
 	  return err;
 	}
       else if (linelen > 2
@@ -789,6 +805,8 @@ status_handler (void *opaque, int fd)
 		    dst++;
 		}
 	    }
+          DEBUG2 ("fd %d: D line; final status: %s\n",
+                  fd, err? gpg_strerror (err):"ok");
         }
       else if (linelen > 2
 	       && line[0] == 'S' && line[1] == ' ')
@@ -811,6 +829,8 @@ status_handler (void *opaque, int fd)
 	    }
 	  else
 	    fprintf (stderr, "[UNKNOWN STATUS]%s %s", line + 2, rest);
+          DEBUG3 ("fd %d: S line (%s) - final status: %s\n",
+                  fd, line+2, err? gpg_strerror (err):"ok");
 	}
     }
   while (!err && assuan_pending_line (gpgsm->assuan_ctx));
@@ -850,7 +870,7 @@ start (engine_gpgsm_t gpgsm, const char *command)
     err = add_io_cb (gpgsm, &gpgsm->message_cb, _gpgme_data_outbound_handler);
 
   if (!err)
-    err = assuan_write_line (gpgsm->assuan_ctx, command);
+    err = map_assuan_error (assuan_write_line (gpgsm->assuan_ctx, command));
 
   if (!err)
     (*gpgsm->io_cbs.event) (gpgsm->io_cbs.event_priv, GPGME_EVENT_START, NULL);
@@ -1108,7 +1128,6 @@ gpgsm_export_ext (void *engine, const char *pattern[], unsigned int reserved,
 	      patlet++;
 	    }
 	  pat++;
-	  /* This will allocate one byte more than necessary.  */
 	  length++;
 	}
     }
@@ -1151,6 +1170,8 @@ gpgsm_export_ext (void *engine, const char *pattern[], unsigned int reserved,
 	      patlet++;
 	    }
 	  pattern++;
+          if (*pattern)
+            *linep++ = ' ';
 	}
     }
   *linep = '\0';
@@ -1242,6 +1263,16 @@ gpgsm_keylist (void *engine, const char *pattern, int secret_only,
   if (err)
     return err;
 
+
+  /* Use the validation mode if required.  We don't check for an error
+     yet because this is a pretty fresh gpgsm features. */
+  gpgsm_assuan_simple_command (gpgsm->assuan_ctx, 
+                               (mode & GPGME_KEYLIST_MODE_VALIDATE)?
+                               "OPTION with-validation=1":
+                               "OPTION with-validation=0" ,
+                               NULL, NULL);
+
+
   /* Length is "LISTSECRETKEYS " + p + '\0'.  */
   line = malloc (15 + strlen (pattern) + 1);
   if (!line)
@@ -1277,6 +1308,7 @@ gpgsm_keylist_ext (void *engine, const char *pattern[], int secret_only,
   /* Length is "LISTSECRETKEYS " + p + '\0'.  */
   int length = 15 + 1;
   char *linep;
+  int any_pattern = 0;
   int list_mode = 0;
 
   if (reserved)
@@ -1294,6 +1326,15 @@ gpgsm_keylist_ext (void *engine, const char *pattern[], int secret_only,
   if (err)
     return err;
 
+  /* Use the validation mode if required.  We don't check for an error
+     yet because this is a pretty fresh gpgsm features. */
+  gpgsm_assuan_simple_command (gpgsm->assuan_ctx, 
+                               (mode & GPGME_KEYLIST_MODE_VALIDATE)?
+                               "OPTION with-validation=1":
+                               "OPTION with-validation=0" ,
+                               NULL, NULL);
+
+
   if (pattern && *pattern)
     {
       const char **pat = pattern;
@@ -1310,7 +1351,6 @@ gpgsm_keylist_ext (void *engine, const char *pattern[], int secret_only,
 	      patlet++;
 	    }
 	  pat++;
-	  /* This will allocate one byte more than necessary.  */
 	  length++;
 	}
     }
@@ -1359,9 +1399,13 @@ gpgsm_keylist_ext (void *engine, const char *pattern[], int secret_only,
 		}
 	      patlet++;
 	    }
+          any_pattern = 1;
+          *linep++ = ' ';
 	  pattern++;
 	}
     }
+  if (any_pattern)
+    linep--;
   *linep = '\0';
 
   _gpgme_io_close (gpgsm->input_cb.fd);
@@ -1395,8 +1439,8 @@ gpgsm_sign (void *engine, gpgme_data_t in, gpgme_data_t out,
   if (err)
     return err;
 
-  /* We must do a reset becuase we need to reset the list of signers.  Note
-     that RESET does not reset OPTION commands. */
+  /* We must send a reset because we need to reset the list of
+     signers.  Note that RESET does not reset OPTION commands. */
   err = gpgsm_assuan_simple_command (gpgsm->assuan_ctx, "RESET", NULL, NULL);
   if (err)
     return err;
@@ -1546,5 +1590,6 @@ struct engine_ops _gpgme_engine_ops_gpgsm =
     NULL,		/* trustlist */
     gpgsm_verify,
     gpgsm_set_io_cbs,
-    gpgsm_io_event
+    gpgsm_io_event,
+    gpgsm_cancel
   };
