@@ -1,24 +1,23 @@
 /* w32-util.c - Utility functions for the W32 API
- *      Copyright (C) 1999 Free Software Foundation, Inc
- *	Copyright (C) 2001 Werner Koch (dd9jn)
- *      Copyright (C) 2001, 2002 g10 Code GmbH
- *
- * This file is part of GPGME.
- *
- * GPGME is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * GPGME is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
- */
+   Copyright (C) 1999 Free Software Foundation, Inc
+   Copyright (C) 2001 Werner Koch (dd9jn)
+   Copyright (C) 2001, 2002, 2003 g10 Code GmbH
+
+   This file is part of GPGME.
+ 
+   GPGME is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
+ 
+   GPGME is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   General Public License for more details.
+ 
+   You should have received a copy of the GNU General Public License
+   along with GPGME; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -36,6 +35,7 @@
 #include <io.h>
 
 #include "util.h"
+#include "sema.h"
 
 DEFINE_STATIC_LOCK (get_path_lock);
 
@@ -73,12 +73,12 @@ read_w32_registry_string (const char *root, const char *dir, const char *name)
   if (RegQueryValueEx (key_handle, name, 0, NULL, NULL, &nbytes))
     goto leave;
   n1 = nbytes + 1;
-  result = xtrymalloc (n1);
+  result = malloc (n1);
   if (!result)
     goto leave;
   if (RegQueryValueEx (key_handle, name, 0, NULL, result, &n1))
     {
-      xfree(result);
+      free (result);
       result = NULL;
       goto leave;
     }
@@ -100,7 +100,7 @@ find_program_in_registry (const char *name)
     {
       int i;
 
-      DEBUG1 ("found %s in registry: `%s'", name, program);
+      DEBUG2 ("found %s in registry: `%s'", name, program);
       for (i = 0; program[i]; i++)
 	{
 	  if (program[i] == '/')
