@@ -18,11 +18,18 @@
    along with GPGME; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
+/* We need to include config.h so that we know whether we are building
+   with large file system (LFS) support. */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
 #include <errno.h>
+#include <sys/types.h>
 #include <sys/select.h>
 
 #include <gpgme.h>
@@ -168,6 +175,8 @@ main (int argc, char *argv[])
   gpgme_key_t key[3] = { NULL, NULL, NULL };
   int i;
 
+  init_gpgme (GPGME_PROTOCOL_OpenPGP);
+
   for (i = 0; i < FDLIST_MAX; i++)
     fdlist[i].fd = -1;
 
